/* ***** BEGIN LICENSE BLOCK *****
 * Copyright (c) 2002-4 Simon Woodside.
 * Original Author: Simon Woodside <sbwoodside@yahoo.com>
 * This file is part of FractalTrees.
 *     FractalTrees is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *     FractalTrees is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *     You should have received a copy of the GNU General Public License
 * along with FractalTrees; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * ***** END LICENSE BLOCK ***** */

#import <AppKit/AppKit.h>

@class FractalBranch;
@class RandSupply;
@class FractalController;

@interface FractalTree : NSObject
{
  FractalController * controller;
  NSMutableArray * branchesArray;
  RandSupply * rand;  // real random numbers from /dev/random
  NSConditionLock * countsLock;
  NSDictionary * counts;
  
  float lengthVariance;
  float angleVariance;
  float baseLength;
  NSPoint basePoint;
  int  maxHeight;
}

- (id)initWithHeight:(int)height controller:(FractalController*)cont;
- (void)createRandomValues:(id)theObject;
- (void)createRandomByRecursing:(FractalBranch *)newBranch length:(float)length angle:(float)angle level:(int)level;

- (FractalBranch *)branchReadyForDrawAtLevel:(int)level index:(int)index;

- (NSConditionLock*)countsLock;
- (void)openLockForUse;

- (NSPoint)basePoint;
- (void)setBasePoint:(NSPoint)base;
- (void)setVarianceLength:(float)length angle:(float)angle;

@end
